@@ -13,82 +13,85 @@ import (
 
 // actionflags.go contains flag variables for action-like commands to draw from
 var (
-	AppName            string
-	BindPaths          []string
-	Mounts             []string
-	HomePath           string
-	OverlayPath        []string
-	ScratchPath        []string
-	WorkdirPath        string
-	PwdPath            string
-	ShellPath          string
-	Hostname           string
-	Network            string
-	NetworkArgs        []string
-	DNS                string
-	Security           []string
-	CgroupsTOMLFile    string
-	VMRAM              string
-	VMCPU              string
-	VMIP               string
-	ContainLibsPath    []string
-	FuseMount          []string
-	SingularityEnv     map[string]string
-	SingularityEnvFile string
-	NoMount            []string
+	appName            string
+	bindPaths          []string
+	mounts             []string
+	homePath           string
+	overlayPath        []string
+	scratchPath        []string
+	workdirPath        string
+	pwdPath            string
+	shellPath          string
+	hostname           string
+	network            string
+	networkArgs        []string
+	dns                string
+	security           []string
+	cgroupsTOMLFile    string
+	vmRAM              string
+	vmCPU              string
+	vmIP               string
+	containLibsPath    []string
+	fuseMount          []string
+	singularityEnv     map[string]string
+	singularityEnvFile string
+	noMount            []string
+	proot              string
 
-	IsBoot          bool
-	IsFakeroot      bool
-	IsCleanEnv      bool
-	IsCompat        bool
-	IsContained     bool
-	IsContainAll    bool
-	IsWritable      bool
-	IsWritableTmpfs bool
-	SIFFUSE         bool
-	Nvidia          bool
-	NvCCLI          bool
-	Rocm            bool
-	NoEval          bool
-	NoHome          bool
-	NoInit          bool
-	NoNvidia        bool
-	NoRocm          bool
-	NoUmask         bool
-	VM              bool
-	VMErr           bool
-	IsSyOS          bool
+	isBoot          bool
+	isFakeroot      bool
+	isCleanEnv      bool
+	isCompat        bool
+	isContained     bool
+	isContainAll    bool
+	isWritable      bool
+	isWritableTmpfs bool
+	sifFUSE         bool
+	nvidia          bool
+	nvCCLI          bool
+	rocm            bool
+	noEval          bool
+	noHome          bool
+	noInit          bool
+	noNvidia        bool
+	noRocm          bool
+	noUmask         bool
+	vm              bool
+	vmErr           bool
+	isSyOS          bool
 	disableCache    bool
 
-	NetNamespace  bool
-	UtsNamespace  bool
-	UserNamespace bool
-	PidNamespace  bool
-	IpcNamespace  bool
+	netNamespace  bool
+	utsNamespace  bool
+	userNamespace bool
+	pidNamespace  bool
+	ipcNamespace  bool
 
-	AllowSUID bool
-	KeepPrivs bool
-	NoPrivs   bool
-	AddCaps   string
-	DropCaps  string
+	allowSUID bool
+	keepPrivs bool
+	noPrivs   bool
+	addCaps   string
+	dropCaps  string
 
-	BlkioWeight       int
-	BlkioWeightDevice []string
-	CPUShares         int
-	CPUs              string // decimal
-	CPUSetCPUs        string
-	CPUSetMems        string
-	Memory            string // bytes
-	MemoryReservation string // bytes
-	MemorySwap        string // bytes
-	OomKillDisable    bool
-	PidsLimit         int
+	blkioWeight       int
+	blkioWeightDevice []string
+	cpuShares         int
+	cpus              string // decimal
+	cpuSetCPUs        string
+	cpuSetMems        string
+	memory            string // bytes
+	memoryReservation string // bytes
+	memorySwap        string // bytes
+	oomKillDisable    bool
+	pidsLimit         int
+
+	ociRuntime bool
 )
 
 // --app
 var actionAppFlag = cmdline.Flag{
 	ID:           "actionAppFlag",
-	Value:        &AppName,
+	Value:        &appName,
 	DefaultValue: "",
 	Name:         "app",
 	Usage:        "set an application to run inside a container",
@@ -98,7 +101,7 @@ var actionAppFlag = cmdline.Flag{
 // -B|--bind
 var actionBindFlag = cmdline.Flag{
 	ID:           "actionBindFlag",
-	Value:        &BindPaths,
+	Value:        &bindPaths,
 	DefaultValue: []string{},
 	Name:         "bind",
 	ShortHand:    "B",
@@ -111,7 +114,7 @@ var actionBindFlag = cmdline.Flag{
 // --mount
 var actionMountFlag = cmdline.Flag{
 	ID:           "actionMountFlag",
-	Value:        &Mounts,
+	Value:        &mounts,
 	DefaultValue: []string{},
 	Name:         "mount",
 	Usage:        "a mount specification e.g. 'type=bind,source=/opt,destination=/hostopt'.",
@@ -124,7 +127,7 @@ var actionMountFlag = cmdline.Flag{
 // -H|--home
 var actionHomeFlag = cmdline.Flag{
 	ID:           "actionHomeFlag",
-	Value:        &HomePath,
+	Value:        &homePath,
 	DefaultValue: CurrentUser.HomeDir,
 	Name:         "home",
 	ShortHand:    "H",
@@ -136,7 +139,7 @@ var actionHomeFlag = cmdline.Flag{
 // -o|--overlay
 var actionOverlayFlag = cmdline.Flag{
 	ID:           "actionOverlayFlag",
-	Value:        &OverlayPath,
+	Value:        &overlayPath,
 	DefaultValue: []string{},
 	Name:         "overlay",
 	ShortHand:    "o",
@@ -148,7 +151,7 @@ var actionOverlayFlag = cmdline.Flag{
 // -S|--scratch
 var actionScratchFlag = cmdline.Flag{
 	ID:           "actionScratchFlag",
-	Value:        &ScratchPath,
+	Value:        &scratchPath,
 	DefaultValue: []string{},
 	Name:         "scratch",
 	ShortHand:    "S",
@@ -160,7 +163,7 @@ var actionScratchFlag = cmdline.Flag{
 // -W|--workdir
 var actionWorkdirFlag = cmdline.Flag{
 	ID:           "actionWorkdirFlag",
-	Value:        &WorkdirPath,
+	Value:        &workdirPath,
 	DefaultValue: "",
 	Name:         "workdir",
 	ShortHand:    "W",
@@ -182,7 +185,7 @@ var actionDisableCacheFlag = cmdline.Flag{
 // -s|--shell
 var actionShellFlag = cmdline.Flag{
 	ID:           "actionShellFlag",
-	Value:        &ShellPath,
+	Value:        &shellPath,
 	DefaultValue: "",
 	Name:         "shell",
 	ShortHand:    "s",
@@ -194,7 +197,7 @@ var actionShellFlag = cmdline.Flag{
 // --pwd
 var actionPwdFlag = cmdline.Flag{
 	ID:           "actionPwdFlag",
-	Value:        &PwdPath,
+	Value:        &pwdPath,
 	DefaultValue: "",
 	Name:         "pwd",
 	Usage:        "initial working directory for payload process inside the container",
@@ -205,7 +208,7 @@ var actionPwdFlag = cmdline.Flag{
 // --hostname
 var actionHostnameFlag = cmdline.Flag{
 	ID:           "actionHostnameFlag",
-	Value:        &Hostname,
+	Value:        &hostname,
 	DefaultValue: "",
 	Name:         "hostname",
 	Usage:        "set container hostname",
@@ -216,7 +219,7 @@ var actionHostnameFlag = cmdline.Flag{
 // --network
 var actionNetworkFlag = cmdline.Flag{
 	ID:           "actionNetworkFlag",
-	Value:        &Network,
+	Value:        &network,
 	DefaultValue: "bridge",
 	Name:         "network",
 	Usage:        "specify desired network type separated by commas, each network will bring up a dedicated interface inside container",
@@ -227,7 +230,7 @@ var actionNetworkFlag = cmdline.Flag{
 // --network-args
 var actionNetworkArgsFlag = cmdline.Flag{
 	ID:           "actionNetworkArgsFlag",
-	Value:        &NetworkArgs,
+	Value:        &networkArgs,
 	DefaultValue: []string{},
 	Name:         "network-args",
 	Usage:        "specify network arguments to pass to CNI plugins",
@@ -238,7 +241,7 @@ var actionNetworkArgsFlag = cmdline.Flag{
 // --dns
 var actionDNSFlag = cmdline.Flag{
 	ID:           "actionDnsFlag",
-	Value:        &DNS,
+	Value:        &dns,
 	DefaultValue: "",
 	Name:         "dns",
 	Usage:        "list of DNS server separated by commas to add in resolv.conf",
@@ -248,7 +251,7 @@ var actionDNSFlag = cmdline.Flag{
 // --security
 var actionSecurityFlag = cmdline.Flag{
 	ID:           "actionSecurityFlag",
-	Value:        &Security,
+	Value:        &security,
 	DefaultValue: []string{},
 	Name:         "security",
 	Usage:        "enable security features (SELinux, Apparmor, Seccomp)",
@@ -258,7 +261,7 @@ var actionSecurityFlag = cmdline.Flag{
 // --apply-cgroups
 var actionApplyCgroupsFlag = cmdline.Flag{
 	ID:           "actionApplyCgroupsFlag",
-	Value:        &CgroupsTOMLFile,
+	Value:        &cgroupsTOMLFile,
 	DefaultValue: "",
 	Name:         "apply-cgroups",
 	Usage:        "apply cgroups from file for container processes (root only)",
@@ -268,7 +271,7 @@ var actionApplyCgroupsFlag = cmdline.Flag{
 // --vm-ram
 var actionVMRAMFlag = cmdline.Flag{
 	ID:           "actionVMRAMFlag",
-	Value:        &VMRAM,
+	Value:        &vmRAM,
 	DefaultValue: "1024",
 	Name:         "vm-ram",
 	Usage:        "amount of RAM in MiB to allocate to Virtual Machine (implies --vm)",
@@ -279,7 +282,7 @@ var actionVMRAMFlag = cmdline.Flag{
 // --vm-cpu
 var actionVMCPUFlag = cmdline.Flag{
 	ID:           "actionVMCPUFlag",
-	Value:        &VMCPU,
+	Value:        &vmCPU,
 	DefaultValue: "1",
 	Name:         "vm-cpu",
 	Usage:        "number of CPU cores to allocate to Virtual Machine (implies --vm)",
@@ -290,7 +293,7 @@ var actionVMCPUFlag = cmdline.Flag{
 // --vm-ip
 var actionVMIPFlag = cmdline.Flag{
 	ID:           "actionVMIPFlag",
-	Value:        &VMIP,
+	Value:        &vmIP,
 	DefaultValue: "dhcp",
 	Name:         "vm-ip",
 	Usage:        "IP Address to assign for container usage. Defaults to DHCP within bridge network.",
@@ -301,7 +304,7 @@ var actionVMIPFlag = cmdline.Flag{
 // hidden flag to handle SINGULARITY_CONTAINLIBS environment variable
 var actionContainLibsFlag = cmdline.Flag{
 	ID:           "actionContainLibsFlag",
-	Value:        &ContainLibsPath,
+	Value:        &containLibsPath,
 	DefaultValue: []string{},
 	Name:         "containlibs",
 	Hidden:       true,
@@ -311,7 +314,7 @@ var actionContainLibsFlag = cmdline.Flag{
 // --fusemount
 var actionFuseMountFlag = cmdline.Flag{
 	ID:           "actionFuseMountFlag",
-	Value:        &FuseMount,
+	Value:        &fuseMount,
 	DefaultValue: []string{},
 	Name:         "fusemount",
 	Usage:        "A FUSE filesystem mount specification of the form '<type>:<fuse command> <mountpoint>' - where <type> is 'container' or 'host', specifying where the mount will be performed ('container-daemon' or 'host-daemon' will run the FUSE process detached). <fuse command> is the path to the FUSE executable, plus options for the mount. <mountpoint> is the location in the container to which the FUSE mount will be attached. E.g. 'container:sshfs 10.0.0.1:/ /sshfs'. Implies --pid.",
@@ -332,7 +335,7 @@ var actionTmpDirFlag = cmdline.Flag{
 // --boot
 var actionBootFlag = cmdline.Flag{
 	ID:           "actionBootFlag",
-	Value:        &IsBoot,
+	Value:        &isBoot,
 	DefaultValue: false,
 	Name:         "boot",
 	Usage:        "execute /sbin/init to boot container (root only)",
@@ -342,7 +345,7 @@ var actionBootFlag = cmdline.Flag{
 // -f|--fakeroot
 var actionFakerootFlag = cmdline.Flag{
 	ID:           "actionFakerootFlag",
-	Value:        &IsFakeroot,
+	Value:        &isFakeroot,
 	DefaultValue: false,
 	Name:         "fakeroot",
 	ShortHand:    "f",
@@ -353,7 +356,7 @@ var actionFakerootFlag = cmdline.Flag{
 // -e|--cleanenv
 var actionCleanEnvFlag = cmdline.Flag{
 	ID:           "actionCleanEnvFlag",
-	Value:        &IsCleanEnv,
+	Value:        &isCleanEnv,
 	DefaultValue: false,
 	Name:         "cleanenv",
 	ShortHand:    "e",
@@ -364,7 +367,7 @@ var actionCleanEnvFlag = cmdline.Flag{
 // --compat
 var actionCompatFlag = cmdline.Flag{
 	ID:           "actionCompatFlag",
-	Value:        &IsCompat,
+	Value:        &isCompat,
 	DefaultValue: false,
 	Name:         "compat",
 	Usage:        "apply settings for increased OCI/Docker compatibility. Infers --containall, --no-init, --no-umask, --no-eval, --writable-tmpfs.",
@@ -374,7 +377,7 @@ var actionCompatFlag = cmdline.Flag{
 // -c|--contain
 var actionContainFlag = cmdline.Flag{
 	ID:           "actionContainFlag",
-	Value:        &IsContained,
+	Value:        &isContained,
 	DefaultValue: false,
 	Name:         "contain",
 	ShortHand:    "c",
@@ -385,7 +388,7 @@ var actionContainFlag = cmdline.Flag{
 // -C|--containall
 var actionContainAllFlag = cmdline.Flag{
 	ID:           "actionContainAllFlag",
-	Value:        &IsContainAll,
+	Value:        &isContainAll,
 	DefaultValue: false,
 	Name:         "containall",
 	ShortHand:    "C",
@@ -396,7 +399,7 @@ var actionContainAllFlag = cmdline.Flag{
 // --nv
 var actionNvidiaFlag = cmdline.Flag{
 	ID:           "actionNvidiaFlag",
-	Value:        &Nvidia,
+	Value:        &nvidia,
 	DefaultValue: false,
 	Name:         "nv",
 	Usage:        "enable Nvidia support",
@@ -406,7 +409,7 @@ var actionNvidiaFlag = cmdline.Flag{
 // --nvccli
 var actionNvCCLIFlag = cmdline.Flag{
 	ID:           "actionNvCCLIFlag",
-	Value:        &NvCCLI,
+	Value:        &nvCCLI,
 	DefaultValue: false,
 	Name:         "nvccli",
 	Usage:        "use nvidia-container-cli for GPU setup (experimental)",
@@ -416,7 +419,7 @@ var actionNvCCLIFlag = cmdline.Flag{
 // --rocm flag to automatically bind
 var actionRocmFlag = cmdline.Flag{
 	ID:           "actionRocmFlag",
-	Value:        &Rocm,
+	Value:        &rocm,
 	DefaultValue: false,
 	Name:         "rocm",
 	Usage:        "enable experimental Rocm support",
@@ -426,7 +429,7 @@ var actionRocmFlag = cmdline.Flag{
 // -w|--writable
 var actionWritableFlag = cmdline.Flag{
 	ID:           "actionWritableFlag",
-	Value:        &IsWritable,
+	Value:        &isWritable,
 	DefaultValue: false,
 	Name:         "writable",
 	ShortHand:    "w",
@@ -437,7 +440,7 @@ var actionWritableFlag = cmdline.Flag{
 // --writable-tmpfs
 var actionWritableTmpfsFlag = cmdline.Flag{
 	ID:           "actionWritableTmpfsFlag",
-	Value:        &IsWritableTmpfs,
+	Value:        &isWritableTmpfs,
 	DefaultValue: false,
 	Name:         "writable-tmpfs",
 	Usage:        "makes the file system accessible as read-write with non persistent data (with overlay support only)",
@@ -447,7 +450,7 @@ var actionWritableTmpfsFlag = cmdline.Flag{
 // --no-home
 var actionNoHomeFlag = cmdline.Flag{
 	ID:           "actionNoHomeFlag",
-	Value:        &NoHome,
+	Value:        &noHome,
 	DefaultValue: false,
 	Name:         "no-home",
 	Usage:        "do NOT mount users home directory if /home is not the current working directory",
@@ -457,17 +460,17 @@ var actionNoHomeFlag = cmdline.Flag{
 // --no-mount
 var actionNoMountFlag = cmdline.Flag{
 	ID:           "actionNoMountFlag",
-	Value:        &NoMount,
+	Value:        &noMount,
 	DefaultValue: []string{},
 	Name:         "no-mount",
-	Usage:        "disable one or more 'mount xxx' options set in singularity.conf, specify absolute destination path to disable a 'bind path' entry",
+	Usage:        "disable one or more 'mount xxx' options set in singularity.conf, specify absolute destination path to disable a bind path entry, or 'bind-paths' to disable all bind path entries.",
 	EnvKeys:      []string{"NO_MOUNT"},
 }
 
 // --no-init
 var actionNoInitFlag = cmdline.Flag{
 	ID:           "actionNoInitFlag",
-	Value:        &NoInit,
+	Value:        &noInit,
 	DefaultValue: false,
 	Name:         "no-init",
 	Usage:        "do NOT start shim process with --pid",
@@ -477,7 +480,7 @@ var actionNoInitFlag = cmdline.Flag{
 // hidden flag to disable nvidia bindings when 'always use nv = yes'
 var actionNoNvidiaFlag = cmdline.Flag{
 	ID:           "actionNoNvidiaFlag",
-	Value:        &NoNvidia,
+	Value:        &noNvidia,
 	DefaultValue: false,
 	Name:         "no-nv",
 	Hidden:       true,
@@ -487,7 +490,7 @@ var actionNoNvidiaFlag = cmdline.Flag{
 // hidden flag to disable rocm bindings when 'always use rocm = yes'
 var actionNoRocmFlag = cmdline.Flag{
 	ID:           "actionNoRocmFlag",
-	Value:        &NoRocm,
+	Value:        &noRocm,
 	DefaultValue: false,
 	Name:         "no-rocm",
 	Hidden:       true,
@@ -497,7 +500,7 @@ var actionNoRocmFlag = cmdline.Flag{
 // --vm
 var actionVMFlag = cmdline.Flag{
 	ID:           "actionVMFlag",
-	Value:        &VM,
+	Value:        &vm,
 	DefaultValue: false,
 	Name:         "vm",
 	Usage:        "enable VM support",
@@ -507,7 +510,7 @@ var actionVMFlag = cmdline.Flag{
 // --vm-err
 var actionVMErrFlag = cmdline.Flag{
 	ID:           "actionVMErrFlag",
-	Value:        &VMErr,
+	Value:        &vmErr,
 	DefaultValue: false,
 	Name:         "vm-err",
 	Usage:        "enable attaching stderr from VM",
@@ -518,7 +521,7 @@ var actionVMErrFlag = cmdline.Flag{
 // TODO: Keep this in production?
 var actionSyOSFlag = cmdline.Flag{
 	ID:           "actionSyOSFlag",
-	Value:        &IsSyOS,
+	Value:        &isSyOS,
 	DefaultValue: false,
 	Name:         "syos",
 	Usage:        "execute SyOS shell",
@@ -528,7 +531,7 @@ var actionSyOSFlag = cmdline.Flag{
 // -p|--pid
 var actionPidNamespaceFlag = cmdline.Flag{
 	ID:           "actionPidNamespaceFlag",
-	Value:        &PidNamespace,
+	Value:        &pidNamespace,
 	DefaultValue: false,
 	Name:         "pid",
 	ShortHand:    "p",
@@ -539,7 +542,7 @@ var actionPidNamespaceFlag = cmdline.Flag{
 // -i|--ipc
 var actionIpcNamespaceFlag = cmdline.Flag{
 	ID:           "actionIpcNamespaceFlag",
-	Value:        &IpcNamespace,
+	Value:        &ipcNamespace,
 	DefaultValue: false,
 	Name:         "ipc",
 	ShortHand:    "i",
@@ -550,7 +553,7 @@ var actionIpcNamespaceFlag = cmdline.Flag{
 // -n|--net
 var actionNetNamespaceFlag = cmdline.Flag{
 	ID:           "actionNetNamespaceFlag",
-	Value:        &NetNamespace,
+	Value:        &netNamespace,
 	DefaultValue: false,
 	Name:         "net",
 	ShortHand:    "n",
@@ -561,7 +564,7 @@ var actionNetNamespaceFlag = cmdline.Flag{
 // --uts
 var actionUtsNamespaceFlag = cmdline.Flag{
 	ID:           "actionUtsNamespaceFlag",
-	Value:        &UtsNamespace,
+	Value:        &utsNamespace,
 	DefaultValue: false,
 	Name:         "uts",
 	Usage:        "run container in a new UTS namespace",
@@ -571,7 +574,7 @@ var actionUtsNamespaceFlag = cmdline.Flag{
 // -u|--userns
 var actionUserNamespaceFlag = cmdline.Flag{
 	ID:           "actionUserNamespaceFlag",
-	Value:        &UserNamespace,
+	Value:        &userNamespace,
 	DefaultValue: false,
 	Name:         "userns",
 	ShortHand:    "u",
@@ -582,7 +585,7 @@ var actionUserNamespaceFlag = cmdline.Flag{
 // --keep-privs
 var actionKeepPrivsFlag = cmdline.Flag{
 	ID:           "actionKeepPrivsFlag",
-	Value:        &KeepPrivs,
+	Value:        &keepPrivs,
 	DefaultValue: false,
 	Name:         "keep-privs",
 	Usage:        "let root user keep privileges in container (root only)",
@@ -592,7 +595,7 @@ var actionKeepPrivsFlag = cmdline.Flag{
 // --no-privs
 var actionNoPrivsFlag = cmdline.Flag{
 	ID:           "actionNoPrivsFlag",
-	Value:        &NoPrivs,
+	Value:        &noPrivs,
 	DefaultValue: false,
 	Name:         "no-privs",
 	Usage:        "drop all privileges from root user in container)",
@@ -602,7 +605,7 @@ var actionNoPrivsFlag = cmdline.Flag{
 // --add-caps
 var actionAddCapsFlag = cmdline.Flag{
 	ID:           "actionAddCapsFlag",
-	Value:        &AddCaps,
+	Value:        &addCaps,
 	DefaultValue: "",
 	Name:         "add-caps",
 	Usage:        "a comma separated capability list to add",
@@ -612,7 +615,7 @@ var actionAddCapsFlag = cmdline.Flag{
 // --drop-caps
 var actionDropCapsFlag = cmdline.Flag{
 	ID:           "actionDropCapsFlag",
-	Value:        &DropCaps,
+	Value:        &dropCaps,
 	DefaultValue: "",
 	Name:         "drop-caps",
 	Usage:        "a comma separated capability list to drop",
@@ -622,7 +625,7 @@ var actionDropCapsFlag = cmdline.Flag{
 // --allow-setuid
 var actionAllowSetuidFlag = cmdline.Flag{
 	ID:           "actionAllowSetuidFlag",
-	Value:        &AllowSUID,
+	Value:        &allowSUID,
 	DefaultValue: false,
 	Name:         "allow-setuid",
 	Usage:        "allow setuid binaries in container (root only)",
@@ -632,7 +635,7 @@ var actionAllowSetuidFlag = cmdline.Flag{
 // --env
 var actionEnvFlag = cmdline.Flag{
 	ID:           "actionEnvFlag",
-	Value:        &SingularityEnv,
+	Value:        &singularityEnv,
 	DefaultValue: map[string]string{},
 	Name:         "env",
 	Usage:        "pass environment variable to contained process",
@@ -641,7 +644,7 @@ var actionEnvFlag = cmdline.Flag{
 // --env-file
 var actionEnvFileFlag = cmdline.Flag{
 	ID:           "actionEnvFileFlag",
-	Value:        &SingularityEnvFile,
+	Value:        &singularityEnvFile,
 	DefaultValue: "",
 	Name:         "env-file",
 	Usage:        "pass environment variables from file to contained process",
@@ -651,7 +654,7 @@ var actionEnvFileFlag = cmdline.Flag{
 // --no-umask
 var actionNoUmaskFlag = cmdline.Flag{
 	ID:           "actionNoUmask",
-	Value:        &NoUmask,
+	Value:        &noUmask,
 	DefaultValue: false,
 	Name:         "no-umask",
 	Usage:        "do not propagate umask to the container, set default 0022 umask",
@@ -661,7 +664,7 @@ var actionNoUmaskFlag = cmdline.Flag{
 // --no-eval
 var actionNoEvalFlag = cmdline.Flag{
 	ID:           "actionNoEval",
-	Value:        &NoEval,
+	Value:        &noEval,
 	DefaultValue: false,
 	Name:         "no-eval",
 	Usage:        "do not shell evaluate env vars or OCI container CMD/ENTRYPOINT/ARGS",
@@ -671,7 +674,7 @@ var actionNoEvalFlag = cmdline.Flag{
 // --blkio-weight
 var actionBlkioWeightFlag = cmdline.Flag{
 	ID:           "actionBlkioWeight",
-	Value:        &BlkioWeight,
+	Value:        &blkioWeight,
 	DefaultValue: 0,
 	Name:         "blkio-weight",
 	Usage:        "Block IO relative weight in range 10-1000, 0 to disable",
@@ -681,7 +684,7 @@ var actionBlkioWeightFlag = cmdline.Flag{
 // --blkio-weight-device
 var actionBlkioWeightDeviceFlag = cmdline.Flag{
 	ID:           "actionBlkioWeightDevice",
-	Value:        &BlkioWeightDevice,
+	Value:        &blkioWeightDevice,
 	DefaultValue: []string{},
 	Name:         "blkio-weight-device",
 	Usage:        "Device specific block IO relative weight",
@@ -691,7 +694,7 @@ var actionBlkioWeightDeviceFlag = cmdline.Flag{
 // --cpu-shares
 var actionCPUSharesFlag = cmdline.Flag{
 	ID:           "actionCPUShares",
-	Value:        &CPUShares,
+	Value:        &cpuShares,
 	DefaultValue: -1,
 	Name:         "cpu-shares",
 	Usage:        "CPU shares for container",
@@ -701,7 +704,7 @@ var actionCPUSharesFlag = cmdline.Flag{
 // --cpus
 var actionCPUsFlag = cmdline.Flag{
 	ID:           "actionCPUs",
-	Value:        &CPUs,
+	Value:        &cpus,
 	DefaultValue: "",
 	Name:         "cpus",
 	Usage:        "Number of CPUs available to container",
@@ -711,7 +714,7 @@ var actionCPUsFlag = cmdline.Flag{
 // --cpuset-cpus
 var actionCPUsetCPUsFlag = cmdline.Flag{
 	ID:           "actionCPUsetCPUs",
-	Value:        &CPUSetCPUs,
+	Value:        &cpuSetCPUs,
 	DefaultValue: "",
 	Name:         "cpuset-cpus",
 	Usage:        "List of host CPUs available to container",
@@ -721,7 +724,7 @@ var actionCPUsetCPUsFlag = cmdline.Flag{
 // --cpuset-mems
 var actionCPUsetMemsFlag = cmdline.Flag{
 	ID:           "actionCPUsetMems",
-	Value:        &CPUSetMems,
+	Value:        &cpuSetMems,
 	DefaultValue: "",
 	Name:         "cpuset-mems",
 	Usage:        "List of host memory nodes available to container",
@@ -731,7 +734,7 @@ var actionCPUsetMemsFlag = cmdline.Flag{
 // --memory
 var actionMemoryFlag = cmdline.Flag{
 	ID:           "actionMemory",
-	Value:        &Memory,
+	Value:        &memory,
 	DefaultValue: "",
 	Name:         "memory",
 	Usage:        "Memory limit in bytes",
@@ -741,7 +744,7 @@ var actionMemoryFlag = cmdline.Flag{
 // --memory-reservation
 var actionMemoryReservationFlag = cmdline.Flag{
 	ID:           "actionMemoryReservation",
-	Value:        &MemoryReservation,
+	Value:        &memoryReservation,
 	DefaultValue: "",
 	Name:         "memory-reservation",
 	Usage:        "Memory soft limit in bytes",
@@ -751,7 +754,7 @@ var actionMemoryReservationFlag = cmdline.Flag{
 // --memory-swap
 var actionMemorySwapFlag = cmdline.Flag{
 	ID:           "actionMemorySwap",
-	Value:        &MemorySwap,
+	Value:        &memorySwap,
 	DefaultValue: "",
 	Name:         "memory-swap",
 	Usage:        "Swap limit, use -1 for unlimited swap",
@@ -761,7 +764,7 @@ var actionMemorySwapFlag = cmdline.Flag{
 // --oom-kill-disable
 var actionOomKillDisableFlag = cmdline.Flag{
 	ID:           "oomKillDisable",
-	Value:        &OomKillDisable,
+	Value:        &oomKillDisable,
 	DefaultValue: false,
 	Name:         "oom-kill-disable",
 	Usage:        "Disable OOM killer",
@@ -771,7 +774,7 @@ var actionOomKillDisableFlag = cmdline.Flag{
 // --pids-limit
 var actionPidsLimitFlag = cmdline.Flag{
 	ID:           "actionPidsLimit",
-	Value:        &PidsLimit,
+	Value:        &pidsLimit,
 	DefaultValue: 0,
 	Name:         "pids-limit",
 	Usage:        "Limit number of container PIDs, use -1 for unlimited",
@@ -781,13 +784,34 @@ var actionPidsLimitFlag = cmdline.Flag{
 // --sif-fuse
 var actionSIFFUSEFlag = cmdline.Flag{
 	ID:           "actionSIFFUSE",
-	Value:        &SIFFUSE,
+	Value:        &sifFUSE,
 	DefaultValue: false,
 	Name:         "sif-fuse",
 	Usage:        "attempt FUSE mount of SIF (unprivileged / user namespace only) (experimental)",
 	EnvKeys:      []string{"SIF_FUSE"},
 }
 
+// --proot (hidden)
+var actionProotFlag = cmdline.Flag{
+	ID:           "actionProot",
+	Value:        &proot,
+	DefaultValue: "",
+	Name:         "proot",
+	Usage:        "Bind proot from the host into /.singularity.d/libs",
+	EnvKeys:      []string{"PROOT"},
+	Hidden:       true,
+}
+
+// --oci
+var actionOCIFlag = cmdline.Flag{
+	ID:           "actionOCI",
+	Value:        &ociRuntime,
+	DefaultValue: false,
+	Name:         "oci",
+	Usage:        "Launch container with OCI runtime (experimental)",
+	EnvKeys:      []string{"OCI"},
+}
+
 func init() {
 	addCmdInit(func(cmdManager *cmdline.CommandManager) {
 		cmdManager.RegisterCmd(ExecCmd)
@@ -861,6 +885,7 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&commonNoHTTPSFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&commonOldNoHTTPSFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&dockerLoginFlag, actionsInstanceCmd...)
+		cmdManager.RegisterFlagForCmd(&dockerHostFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&dockerPasswordFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&dockerUsernameFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionEnvFlag, actionsInstanceCmd...)
@@ -879,5 +904,7 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&actionOomKillDisableFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionPidsLimitFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionSIFFUSEFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&actionProotFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&actionOCIFlag, actionsCmd...)
 	})
 }
