@@ -89,7 +89,7 @@ var inspectAppsListFlag = cmdline.Flag{
 // --app
 var inspectAppNameFlag = cmdline.Flag{
 	ID:           "inspectAppNameFlag",
-	Value:        &AppName,
+	Value:        &appName,
 	DefaultValue: "",
 	Name:         "app",
 	Usage:        "inspect a specific app",
@@ -634,10 +634,10 @@ var InspectCmd = &cobra.Command{
 		if allData {
 			// display all data in JSON format only
 			jsonfmt = true
-			AppName = ""
+			appName = ""
 		}
 
-		inspectCmd := newCommand(allData, AppName, img)
+		inspectCmd := newCommand(allData, appName, img)
 
 		// Try to inspect the label partition, if not, then exec/shell
 		// the container to get the data.
@@ -665,7 +665,7 @@ var InspectCmd = &cobra.Command{
 		}
 
 		if startscript || allData {
-			if AppName == "" {
+			if appName == "" {
 				sylog.Debugf("Inspection of startscript selected.")
 				inspectCmd.addStartscriptCommand()
 			}
@@ -691,7 +691,7 @@ var InspectCmd = &cobra.Command{
 		}
 
 		for app := range inspectData.Data.Attributes.Apps {
-			if !listApps && !allData && AppName != app {
+			if !listApps && !allData && appName != app {
 				delete(inspectData.Data.Attributes.Apps, app)
 			}
 		}
@@ -704,7 +704,7 @@ var InspectCmd = &cobra.Command{
 			}
 			fmt.Printf("%s\n", string(jsonObj))
 		} else {
-			appAttr := inspectData.Data.Attributes.Apps[AppName]
+			appAttr := inspectData.Data.Attributes.Apps[appName]
 
 			if listApps {
 				printSortedApp(inspectData.Data.Attributes.Apps)
