@@ -12,6 +12,7 @@ import (
 	"os"
 	"path"
 	"path/filepath"
+	"strings"
 	"testing"
 	"time"
 
@@ -353,9 +354,11 @@ func (c actionTests) issue5211(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("CheckRoot"),
 		e2e.WithProfile(e2e.FakerootProfile),
 		e2e.WithDir(u.Dir),
 		e2e.WithCommand("exec"),
+		e2e.WithGlobalOptions("--debug"),
 		e2e.WithArgs("--contain", c.env.ImagePath, "test", "!", "-d", filepath.Join("/root", canaryBasename)),
 		e2e.ExpectExit(0),
 	)
@@ -363,9 +366,11 @@ func (c actionTests) issue5211(t *testing.T) {
 	// Check we preserve `$HOME` as /root even when we `--contain` with `--fakeroot`
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("CheckHome"),
 		e2e.WithProfile(e2e.FakerootProfile),
 		e2e.WithDir(u.Dir),
 		e2e.WithCommand("exec"),
+		e2e.WithGlobalOptions("--debug"),
 		e2e.WithArgs("--contain", c.env.ImagePath, "sh", "-c", "echo $HOME"),
 		e2e.ExpectExit(
 			0,
@@ -389,7 +394,7 @@ func (c actionTests) issue5271(t *testing.T) {
 	)
 }
 
-// Check that we get a warning when using --writable-tmpfs with underlay.
+// Check that we get a warning when using --writable-tmpfs if unpriv overlay not supported.
 func (c actionTests) issue5307(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
@@ -397,11 +402,22 @@ func (c actionTests) issue5307(t *testing.T) {
 		t,
 		e2e.WithProfile(e2e.UserNamespaceProfile),
 		e2e.WithCommand("exec"),
-		e2e.WithArgs("--writable-tmpfs", c.env.ImagePath, "true"),
-		e2e.ExpectExit(
-			0,
-			e2e.ExpectError(e2e.ContainMatch, "Disabling --writable-tmpfs"),
-		),
+		e2e.WithArgs("--writable-tmpfs", c.env.ImagePath, "mount"),
+		e2e.ExpectExit(0, func(t *testing.T, r *e2e.SingularityCmdResult) {
+			out := string(r.Stdout)
+			err := string(r.Stderr)
+
+			if strings.Contains(out, "overlay") {
+				if strings.Contains(err, "Disabling --writable-tmpfs") {
+					t.Errorf("Overlay used. Unexpected \"--Disabling --writable-tmpfs\", found in output: %s", err)
+				}
+				return
+			}
+
+			if !strings.Contains(err, "Disabling --writable-tmpfs") {
+				t.Errorf("Overlay not used. Expected \"--Disabling --writable-tmpfs\", but not found in output: %s", err)
+			}
+		}),
 	)
 }
 
