@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2022 Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2023 Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -15,10 +15,12 @@ package docker
 import (
 	"fmt"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"strings"
 	"testing"
 
+	dockerclient "github.com/docker/docker/client"
 	"github.com/pkg/errors"
 	"github.com/sylabs/singularity/e2e/internal/e2e"
 	"github.com/sylabs/singularity/e2e/internal/testhelper"
@@ -51,37 +53,24 @@ func (c ctx) testDockerPulls(t *testing.T) {
 		exit    int
 	}{
 		{
-			name:  "AlpineLatestPull",
+			name:  "BusyboxLatestPull",
 			image: tmpImage,
-			uri:   "docker://alpine:latest",
+			uri:   "docker://busybox:latest",
 			exit:  0,
 		},
-		{
-			name:  "Alpine3.9Pull",
-			image: filepath.Join(tmpPath, "alpine.sif"),
-			uri:   "docker://alpine:3.9",
-			exit:  0,
-		},
-		{
-			name:    "Alpine3.9ForcePull",
-			options: []string{"--force"},
-			image:   tmpImage,
-			uri:     "docker://alpine:3.9",
-			exit:    0,
-		},
-		{
-			name:    "BusyboxLatestPull",
-			options: []string{"--force"},
-			image:   tmpImage,
-			uri:     "docker://busybox:latest",
-			exit:    0,
-		},
 		{
 			name:  "BusyboxLatestPullFail",
 			image: tmpImage,
 			uri:   "docker://busybox:latest",
 			exit:  255,
 		},
+		{
+			name:    "BusyboxLatestPullForce",
+			options: []string{"--force"},
+			image:   tmpImage,
+			uri:     "docker://busybox:latest",
+			exit:    0,
+		},
 		{
 			name:    "Busybox1.28Pull",
 			options: []string{"--force", "--dir", tmpPath},
@@ -117,7 +106,7 @@ func (c ctx) testDockerPulls(t *testing.T) {
 					if path == tmpContainerFile {
 						path = filepath.Join(tmpPath, tmpContainerFile)
 					}
-					c.env.ImageVerify(t, path, e2e.UserProfile)
+					c.env.ImageVerify(t, path)
 				}
 			}),
 			e2e.ExpectExit(tt.exit),
@@ -125,6 +114,148 @@ func (c ctx) testDockerPulls(t *testing.T) {
 	}
 }
 
+// Testing DOCKER_ host support (only if docker available)
+func (c ctx) testDockerHost(t *testing.T) {
+	require.Command(t, "docker")
+
+	// Create a Dockerfile for a small image we can build locally
+	tmpPath, err := fs.MakeTmpDir(c.env.TestDir, "docker-", 0o755)
+	err = errors.Wrapf(err, "creating temporary directory in %q for docker host test", c.env.TestDir)
+	if err != nil {
+		t.Fatalf("failed to create temporary directory: %+v", err)
+	}
+	defer os.RemoveAll(tmpPath)
+
+	dockerfile := filepath.Join(tmpPath, "Dockerfile")
+	dockerfileContent := []byte("FROM alpine:latest\n")
+	err = os.WriteFile(dockerfile, dockerfileContent, 0o644)
+	if err != nil {
+		t.Fatalf("failed to create temporary Dockerfile: %+v", err)
+	}
+
+	dockerRef := "dinosaur/test-image:latest"
+	dockerURI := "docker-daemon:" + dockerRef
+
+	// Invoke docker build to build image in the docker daemon.
+	// Use os/exec because easier to generate a command with a working directory
+	e2e.Privileged(func(t *testing.T) {
+		cmd := exec.Command("docker", "build", "-t", dockerRef, tmpPath)
+		cmd.Dir = tmpPath
+		out, err := cmd.CombinedOutput()
+		t.Log(cmd.Args)
+		if err != nil {
+			t.Fatalf("Unexpected error while running command.\n%s: %s", err, string(out))
+		}
+	})(t)
+
+	tests := []struct {
+		name       string
+		envarName  string
+		envarValue string
+		exit       int
+	}{
+		// Unset docker host should use default and succeed
+		{
+			name:       "singularityDockerHostEmpty",
+			envarName:  "SINGULARITY_DOCKER_HOST",
+			envarValue: "",
+			exit:       0,
+		},
+		{
+			name:       "dockerHostEmpty",
+			envarName:  "DOCKER_HOST",
+			envarValue: "",
+			exit:       0,
+		},
+
+		// bad Docker host should fail
+		{
+			name:       "singularityDockerHostInvalid",
+			envarName:  "SINGULARITY_DOCKER_HOST",
+			envarValue: "tcp://192.168.59.103:oops",
+			exit:       255,
+		},
+		{
+			name:       "dockerHostInvalid",
+			envarName:  "DOCKER_HOST",
+			envarValue: "tcp://192.168.59.103:oops",
+			exit:       255,
+		},
+
+		// Set to default should succeed
+		// The default host varies based on OS, so we use dockerclient default
+		{
+			name:       "singularityDockerHostValid",
+			envarName:  "SINGULARITY_DOCKER_HOST",
+			envarValue: dockerclient.DefaultDockerHost,
+			exit:       0,
+		},
+		{
+			name:       "dockerHostValid",
+			envarName:  "DOCKER_HOST",
+			envarValue: dockerclient.DefaultDockerHost,
+			exit:       0,
+		},
+	}
+
+	for _, profile := range []e2e.Profile{e2e.RootProfile, e2e.OCIRootProfile} {
+		t.Run(profile.String(), func(t *testing.T) {
+			t.Run("exec", func(t *testing.T) {
+				for _, tt := range tests {
+					cmdOps := []e2e.SingularityCmdOp{
+						e2e.WithProfile(profile),
+						e2e.AsSubtest(profile.String() + "/" + tt.name),
+						e2e.WithCommand("exec"),
+						e2e.WithArgs("--disable-cache", dockerURI, "/bin/true"),
+						e2e.WithEnv(append(os.Environ(), tt.envarName+"="+tt.envarValue)),
+						e2e.ExpectExit(tt.exit),
+					}
+					c.env.RunSingularity(t, cmdOps...)
+				}
+			})
+
+			t.Run("pull", func(t *testing.T) {
+				for _, tt := range tests {
+					cmdOps := []e2e.SingularityCmdOp{
+						e2e.WithProfile(profile),
+						e2e.AsSubtest(tt.name),
+						e2e.WithCommand("pull"),
+						e2e.WithArgs("--force", "--disable-cache", dockerURI),
+						e2e.WithEnv(append(os.Environ(), tt.envarName+"="+tt.envarValue)),
+						e2e.WithDir(tmpPath),
+						e2e.ExpectExit(tt.exit),
+					}
+					c.env.RunSingularity(t, cmdOps...)
+				}
+			})
+
+			t.Run("build", func(t *testing.T) {
+				for _, tt := range tests {
+					cmdOps := []e2e.SingularityCmdOp{
+						e2e.WithProfile(profile),
+						e2e.AsSubtest(tt.name),
+						e2e.WithCommand("build"),
+						e2e.WithArgs("--force", "--disable-cache", "test.sif", dockerURI),
+						e2e.WithEnv(append(os.Environ(), tt.envarName+"="+tt.envarValue)),
+						e2e.WithDir(tmpPath),
+						e2e.ExpectExit(tt.exit),
+					}
+					c.env.RunSingularity(t, cmdOps...)
+				}
+			})
+		})
+	}
+
+	// Clean up docker image
+	e2e.Privileged(func(t *testing.T) {
+		cmd := exec.Command("docker", "rmi", dockerRef)
+		_, err = cmd.Output()
+		if err != nil {
+			t.Fatalf("Unexpected error while cleaning up docker image.\n%s", err)
+		}
+	})(t)
+}
+
 // AUFS sanity tests
 func (c ctx) testDockerAUFS(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "aufs-", "")
@@ -233,7 +364,7 @@ func (c ctx) testDockerWhiteoutSymlink(t *testing.T) {
 			if t.Failed() {
 				return
 			}
-			c.env.ImageVerify(t, imagePath, e2e.UserProfile)
+			c.env.ImageVerify(t, imagePath)
 		}),
 		e2e.ExpectExit(0),
 	)
@@ -267,44 +398,20 @@ func (c ctx) testDockerDefFile(t *testing.T) {
 		archRequired        string
 		from                string
 	}{
-		// This only provides Arch for amd64
 		{
-			name:                "Arch",
+			name:                "Alpine",
+			kernelMajorRequired: 0,
+			from:                "alpine:latest",
+		},
+		{
+			name:                "AlmaLinux_9",
 			kernelMajorRequired: 3,
-			archRequired:        "amd64",
-			from:                "dock0/arch:latest",
+			from:                "almalinux:9",
 		},
 		{
-			name:                "BusyBox",
-			kernelMajorRequired: 0,
-			from:                "busybox:latest",
-		},
-		// CentOS6 is for amd64 (or i386) only
-		{
-			name:                "CentOS_6",
-			kernelMajorRequired: 0,
-			archRequired:        "amd64",
-			from:                "centos:6",
-		},
-		{
-			name:                "CentOS_7",
-			kernelMajorRequired: 0,
-			from:                "centos:7",
-		},
-		{
-			name:                "AlmaLinux_8",
+			name:                "Ubuntu_2204",
 			kernelMajorRequired: 3,
-			from:                "almalinux:8",
-		},
-		{
-			name:                "Ubuntu_1604",
-			kernelMajorRequired: 0,
-			from:                "ubuntu:16.04",
-		},
-		{
-			name:                "Ubuntu_1804",
-			kernelMajorRequired: 3,
-			from:                "ubuntu:18.04",
+			from:                "ubuntu:22.04",
 		},
 	}
 
@@ -334,7 +441,7 @@ func (c ctx) testDockerDefFile(t *testing.T) {
 					return
 				}
 
-				c.env.ImageVerify(t, imagePath, e2e.RootProfile)
+				c.env.ImageVerify(t, imagePath)
 			}),
 			e2e.ExpectExit(0),
 		)
@@ -346,8 +453,6 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 	defer cleanup(t)
 	imagePath := filepath.Join(imageDir, "container")
 
-	e2e.EnsureRegistry(t)
-
 	tests := []struct {
 		name string
 		exit int
@@ -358,7 +463,7 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 			exit: 0,
 			dfd: e2e.DefFileDetails{
 				Bootstrap: "docker",
-				From:      "localhost:5000/my-busybox",
+				From:      c.env.TestRegistry + "/my-busybox",
 			},
 		},
 		{
@@ -367,7 +472,7 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 			dfd: e2e.DefFileDetails{
 				Bootstrap: "docker",
 				From:      "my-busybox",
-				Registry:  "localhost:5000",
+				Registry:  c.env.TestRegistry,
 			},
 		},
 		{
@@ -376,7 +481,7 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 			dfd: e2e.DefFileDetails{
 				Bootstrap: "docker",
 				From:      "my-busybox",
-				Registry:  "localhost:5000",
+				Registry:  c.env.TestRegistry,
 				Namespace: "not-a-namespace",
 			},
 		},
@@ -389,7 +494,7 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 			t,
 			e2e.WithProfile(e2e.RootProfile),
 			e2e.WithCommand("build"),
-			e2e.WithArgs("--no-https", imagePath, defFile),
+			e2e.WithArgs("--disable-cache", "--no-https", imagePath, defFile),
 			e2e.PostRun(func(t *testing.T) {
 				defer os.Remove(imagePath)
 				defer os.Remove(defFile)
@@ -398,7 +503,7 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 					return
 				}
 
-				c.env.ImageVerify(t, imagePath, e2e.RootProfile)
+				c.env.ImageVerify(t, imagePath)
 			}),
 			e2e.ExpectExit(tt.exit),
 		)
@@ -729,6 +834,54 @@ func (c ctx) testDockerCMDQuotes(t *testing.T) {
 	)
 }
 
+// Check that the USER in a docker container is honored under --oci mode
+func (c ctx) testDockerUSER(t *testing.T) {
+	tests := []struct {
+		name         string
+		expectOutput string
+		profile      e2e.Profile
+	}{
+		// Sanity check singularity native engine... no support for USER
+		{
+			name:    "default",
+			profile: e2e.UserProfile,
+			expectOutput: fmt.Sprintf("uid=%d(%s) gid=%d",
+				e2e.UserProfile.ContainerUser(t).UID,
+				e2e.UserProfile.ContainerUser(t).Name,
+				e2e.UserProfile.ContainerUser(t).GID),
+		},
+		// `--oci` modes (USER honored by default)
+		{
+			name:         "OCIUser",
+			profile:      e2e.OCIUserProfile,
+			expectOutput: `uid=2000(testuser) gid=2000(testgroup)`,
+		},
+		{
+			name:         "OCIFakeroot",
+			profile:      e2e.OCIFakerootProfile,
+			expectOutput: `uid=0(root) gid=0(root)`,
+		},
+		{
+			name:         "OCIRoot",
+			profile:      e2e.OCIRootProfile,
+			expectOutput: `uid=2000(testuser) gid=2000(testgroup)`,
+		},
+	}
+
+	for _, tt := range tests {
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(tt.name),
+			e2e.WithProfile(tt.profile),
+			e2e.WithCommand("run"),
+			e2e.WithArgs("docker://sylabsio/docker-user"),
+			e2e.ExpectExit(0,
+				e2e.ExpectOutput(e2e.ContainMatch, tt.expectOutput),
+			),
+		)
+	}
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := ctx{
@@ -736,7 +889,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	}
 
 	return testhelper.Tests{
-		// Run all docker:// source tests sequentially amongst themselves, so we
+		// Run most docker:// source tests sequentially amongst themselves, so we
 		// don't hit DockerHub massively in parallel, and we benefit from
 		// caching as the same images are used frequently.
 		"ordered": func(t *testing.T) {
@@ -744,18 +897,25 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 			t.Run("def file", c.testDockerDefFile)
 			t.Run("permissions", c.testDockerPermissions)
 			t.Run("pulls", c.testDockerPulls)
-			t.Run("registry", c.testDockerRegistry)
 			t.Run("whiteout symlink", c.testDockerWhiteoutSymlink)
 			t.Run("labels", c.testDockerLabels)
 			t.Run("cmd", c.testDockerCMD)
 			t.Run("entrypoint", c.testDockerENTRYPOINT)
 			t.Run("cmdentrypoint", c.testDockerCMDENTRYPOINT)
 			t.Run("cmd quotes", c.testDockerCMDQuotes)
+			t.Run("user", c.testDockerUSER)
 			// Regressions
 			t.Run("issue 4524", c.issue4524)
-			t.Run("issue 4943", c.issue4943)
-			t.Run("issue 5172", c.issue5172)
-			t.Run("issue 274", c.issue274) // https://github.com/sylabs/singularity/issues/274
+			t.Run("issue 1286", c.issue1286)
 		},
+		// Tests that are especially slow, or run against a local docker
+		// registry, can be run in parallel, with `--disable-cache` used within
+		// them to avoid docker caching concurrency issues.
+		"docker host": c.testDockerHost,
+		"registry":    c.testDockerRegistry,
+		// Regressions
+		"issue 4943": c.issue4943,
+		"issue 5172": c.issue5172,
+		"issue 274":  c.issue274, // https://github.com/sylabs/singularity/issues/274
 	}
 }
