@@ -19,16 +19,6 @@ import (
 	"github.com/sylabs/singularity/pkg/ociruntime"
 )
 
-//  NOTE
-//  ----
-//  Tests in this package/topic are run in a a mount namespace only. There is
-//  no PID namespace, in order that the systemd cgroups manager functionality
-//  can be exercised.
-//
-//  You must take extra care not to leave detached process etc. that will
-//  pollute the host PID namespace.
-//
-
 func randomContainerID(t *testing.T) string {
 	t.Helper()
 
@@ -76,6 +66,7 @@ func genericOciMount(t *testing.T, c *ctx) (string, func()) {
 	}
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("mount"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci mount"),
 		e2e.WithArgs(c.env.ImagePath, bundleDir),
@@ -85,6 +76,7 @@ func genericOciMount(t *testing.T, c *ctx) (string, func()) {
 	cleanup := func() {
 		c.env.RunSingularity(
 			t,
+			e2e.AsSubtest("umount"),
 			e2e.WithProfile(e2e.RootProfile),
 			e2e.WithCommand("oci umount"),
 			e2e.WithArgs(bundleDir),
@@ -141,6 +133,7 @@ func (c ctx) testOciAttach(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("create"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci create"),
 		e2e.WithArgs("-b", bundleDir, containerID),
@@ -159,6 +152,7 @@ func (c ctx) testOciAttach(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("start"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci start"),
 		e2e.WithArgs(containerID),
@@ -172,6 +166,7 @@ func (c ctx) testOciAttach(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("attach"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci attach"),
 		e2e.WithArgs(containerID),
@@ -190,6 +185,7 @@ func (c ctx) testOciAttach(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("delete"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci delete"),
 		e2e.WithArgs(containerID),
@@ -208,6 +204,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("create"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci create"),
 		e2e.WithArgs("-b", bundleDir, containerID),
@@ -226,6 +223,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("start"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci start"),
 		e2e.WithArgs(containerID),
@@ -239,6 +237,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("pause"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci pause"),
 		e2e.WithArgs(containerID),
@@ -256,6 +255,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("resume"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci resume"),
 		e2e.WithArgs(containerID),
@@ -273,6 +273,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("start again"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci start"),
 		e2e.WithArgs(containerID),
@@ -281,6 +282,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("exec"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci exec"),
 		e2e.WithArgs(containerID, "hostname"),
@@ -290,6 +292,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("kill"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci kill"),
 		e2e.WithArgs(containerID, "KILL"),
@@ -303,6 +306,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("delete"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci delete"),
 		e2e.WithArgs(containerID),
@@ -311,6 +315,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("state fail"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci state"),
 		e2e.WithArgs(containerID),
@@ -318,6 +323,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 	)
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("kill fail"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci kill"),
 		e2e.WithArgs(containerID),
@@ -325,6 +331,7 @@ func (c ctx) testOciBasic(t *testing.T) {
 	)
 	c.env.RunSingularity(
 		t,
+		e2e.AsSubtest("start fail"),
 		e2e.WithProfile(e2e.RootProfile),
 		e2e.WithCommand("oci start"),
 		e2e.WithArgs(containerID),
@@ -416,7 +423,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 				t.Run("basic", c.testOciBasic)
 				t.Run("attach", c.testOciAttach)
 				t.Run("run", c.testOciRun)
-				t.Run("help", c.testOciHelp)
 			})),
+		"help": c.testOciHelp,
 	}
 }
