@@ -153,18 +153,18 @@ func create(ctx context.Context, engine *EngineOperations, rpcOps *client.RPC, p
 	}
 
 	// load image driver plugins
-	callbackType := (singularitycallback.RegisterImageDriver)(nil)
+	callbackType := (singularitycallback.RegisterImageDriver)(nil) // nolint:staticcheck
 	callbacks, err := plugin.LoadCallbacks(callbackType)
 	if err != nil {
 		return fmt.Errorf("while loading plugins callbacks '%T': %s", callbackType, err)
 	}
 	for _, callback := range callbacks {
-		if err := callback.(singularitycallback.RegisterImageDriver)(c.userNS); err != nil {
+		if err := callback.(singularitycallback.RegisterImageDriver)(c.userNS); err != nil { // nolint:staticcheck
 			return fmt.Errorf("while registering image driver: %s", err)
 		}
 	}
 
-	driverName := c.engine.EngineConfig.File.ImageDriver
+	driverName := c.engine.EngineConfig.File.ImageDriver // nolint:staticcheck
 	imageDriver = image.GetDriver(driverName)
 	if driverName != "" && imageDriver == nil {
 		return fmt.Errorf("%q: no such image driver", driverName)
@@ -497,7 +497,7 @@ func (c *container) setupImageDriver(system *mount.System) error {
 
 			umountPoints = append(umountPoints, sp)
 
-			sylog.Debugf("Starting image driver %s", c.engine.EngineConfig.File.ImageDriver)
+			sylog.Debugf("Starting image driver %s", c.engine.EngineConfig.File.ImageDriver) // nolint:staticcheck
 			if err := imageDriver.Start(params); err != nil {
 				return fmt.Errorf("failed to start driver: %s", err)
 			}
@@ -511,7 +511,7 @@ func (c *container) setupImageDriver(system *mount.System) error {
 		if params.UsernsFd != -1 {
 			defer unix.Close(params.UsernsFd)
 		}
-		sylog.Debugf("Starting image driver %s", c.engine.EngineConfig.File.ImageDriver)
+		sylog.Debugf("Starting image driver %s", c.engine.EngineConfig.File.ImageDriver) // nolint:staticcheck
 		if err := imageDriver.Start(params); err != nil {
 			return fmt.Errorf("failed to start driver: %s", err)
 		}
@@ -1001,8 +1001,18 @@ func (c *container) addOverlayMount(system *mount.System) error {
 				}
 				ov.AddLowerDir(dst)
 			case image.SANDBOX:
+				// In setuid mode, only root user may mount a directory overlay, as the mount
+				// will take place privileged, and could be abused.
 				allowed := os.Geteuid() == 0
 
+				// In non-setuid mode (user namespace) then we can proceed, as the overlay mount
+				// will be attempted unprivileged. This will succeed if kernel supports it.
+				if c.userNS {
+					allowed = true
+				}
+
+				// If a (deprecated) image driver plugin is enabled, then proceed with the mount.
+				// The image driver will be responsible for handling priv / unpriv concerns.
 				if c.engine.EngineConfig.File.EnableOverlay == "driver" {
 					if imageDriver != nil && imageDriver.Features()&image.OverlayFeature != 0 {
 						allowed = true
@@ -1010,7 +1020,7 @@ func (c *container) addOverlayMount(system *mount.System) error {
 				}
 
 				if !allowed {
-					return fmt.Errorf("only root user can use sandbox as overlay")
+					return fmt.Errorf("only root user can use sandbox as overlay in setuid mode")
 				}
 
 				flags := uintptr(c.suidFlag | syscall.MS_NODEV)
@@ -1439,7 +1449,7 @@ func (c *container) addDevMount(system *mount.System) error {
 		}
 
 		if c.engine.EngineConfig.GetRocm() {
-			devs, err := gpu.RocmDevices(true)
+			devs, err := gpu.RocmDevices()
 			if err != nil {
 				return fmt.Errorf("failed to get rocm devices: %v", err)
 			}
@@ -1527,7 +1537,8 @@ func (c *container) addBindsMount(system *mount.System) error {
 		localtimePath = "/etc/localtime"
 	)
 
-	noBinds := c.engine.EngineConfig.GetNoBinds()
+	skipBinds := c.engine.EngineConfig.GetSkipBinds()
+	skipAllBinds := slice.ContainsString(skipBinds, "*")
 
 	if c.engine.EngineConfig.GetContain() {
 		hosts := hostsPath
@@ -1548,7 +1559,7 @@ func (c *container) addBindsMount(system *mount.System) error {
 			hosts, _ = c.session.GetPath(hostsPath)
 		}
 
-		if !slice.ContainsString(noBinds, hostsPath) {
+		if !skipAllBinds && !slice.ContainsString(skipBinds, hostsPath) {
 			// #5465 If hosts/localtime mount fails, it should not be fatal so skip-on-error
 			if err := system.Points.AddBind(mount.BindsTag, hosts, hostsPath, flags, "skip-on-error"); err != nil {
 				return fmt.Errorf("unable to add %s to mount list: %s", hosts, err)
@@ -1557,7 +1568,7 @@ func (c *container) addBindsMount(system *mount.System) error {
 				return fmt.Errorf("unable to add %s for remount: %s", hostsPath, err)
 			}
 		}
-		if !slice.ContainsString(noBinds, localtimePath) {
+		if !skipAllBinds && !slice.ContainsString(skipBinds, localtimePath) {
 			if err := system.Points.AddBind(mount.BindsTag, localtimePath, localtimePath, flags, "skip-on-error"); err != nil {
 				return fmt.Errorf("unable to add %s to mount list: %s", localtimePath, err)
 			}
@@ -1580,7 +1591,7 @@ func (c *container) addBindsMount(system *mount.System) error {
 
 		sylog.Verbosef("Found 'bind path' = %s, %s", src, dst)
 
-		if slice.ContainsString(noBinds, dst) {
+		if skipAllBinds || slice.ContainsString(skipBinds, dst) {
 			sylog.Debugf("Skipping bind to %s at user request", dst)
 			continue
 		}
