@@ -214,7 +214,7 @@ func TestLoadConfig(t *testing.T) {
 }
 
 func TestValidateConfig(t *testing.T) {
-	dirPath, err := filepath.Abs(filepath.Join("testdata", "images"))
+	dirPath, err := filepath.Abs(filepath.Join("..", "..", "..", "test", "images"))
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -328,7 +328,7 @@ func TestValidateConfig(t *testing.T) {
 func getTestEntity(t *testing.T) *openpgp.Entity {
 	t.Helper()
 
-	f, err := os.Open(filepath.Join("testdata", "keys", "private.asc"))
+	f, err := os.Open(filepath.Join("..", "..", "..", "test", "keys", "pgp-public.asc"))
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -346,7 +346,7 @@ func getTestEntity(t *testing.T) *openpgp.Entity {
 }
 
 func TestShouldRun(t *testing.T) {
-	dirPath, err := filepath.Abs(filepath.Join("testdata", "images"))
+	dirPath, err := filepath.Abs(filepath.Join("..", "..", "..", "test", "images"))
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -393,8 +393,8 @@ func TestShouldRun(t *testing.T) {
 	}
 
 	unsigned := filepath.Join(dirPath, "one-group.sif")
-	signed := filepath.Join(dirPath, "one-group-signed.sif")
-	legacySigned := filepath.Join(dirPath, "one-group-legacy-signed.sif")
+	signed := filepath.Join(dirPath, "one-group-signed-pgp.sif")
+	legacySigned := filepath.Join(dirPath, "one-group-signed-legacy.sif")
 
 	//nolint:maligned // the aligned form, with eg first, is not as easy to read
 	tests := []struct {
