@@ -19,10 +19,7 @@ func TestEnsureFileWithPermission(t *testing.T) {
 	test.DropPrivilege(t)
 	defer test.ResetPrivilege(t)
 
-	tmpDir, err := os.MkdirTemp("", "ensure_file_perm-")
-	if err != nil {
-		t.Errorf("Unable to make tmpdir %s", err)
-	}
+	tmpDir := t.TempDir()
 
 	//
 	// First test: Ensure a already-existing file is the
@@ -132,12 +129,6 @@ func TestEnsureFileWithPermission(t *testing.T) {
 	if currentMode := einfo.Mode(); currentMode != 0o544 {
 		t.Errorf("Unexpected file permission: expecting 544, got %o", currentMode)
 	}
-
-	// Cleanup.
-	err = os.RemoveAll(tmpDir)
-	if err != nil {
-		t.Errorf("Unable to remove tmpdir: %s", err)
-	}
 }
 
 func TestIsFile(t *testing.T) {
@@ -228,11 +219,7 @@ func TestMkdirAll(t *testing.T) {
 	test.DropPrivilege(t)
 	defer test.ResetPrivilege(t)
 
-	tmpdir, err := os.MkdirTemp("", "mkdir")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tmpdir)
+	tmpdir := t.TempDir()
 
 	if err := MkdirAll(filepath.Join(tmpdir, "test"), 0o777); err != nil {
 		t.Error(err)
@@ -256,13 +243,7 @@ func TestMkdir(t *testing.T) {
 	test.DropPrivilege(t)
 	defer test.ResetPrivilege(t)
 
-	tmpdir, err := os.MkdirTemp("", "mkdir")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tmpdir)
-
-	test := filepath.Join(tmpdir, "test")
+	test := filepath.Join(t.TempDir(), "test")
 	if err := Mkdir(test, 0o777); err != nil {
 		t.Error(err)
 	}
@@ -279,11 +260,7 @@ func TestEvalRelative(t *testing.T) {
 	test.DropPrivilege(t)
 	defer test.ResetPrivilege(t)
 
-	tmpdir, err := os.MkdirTemp("", "evalrelative")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tmpdir)
+	tmpdir := t.TempDir()
 
 	// test layout
 	// - /bin -> usr/bin
@@ -358,11 +335,7 @@ func TestTouch(t *testing.T) {
 	test.DropPrivilege(t)
 	defer test.ResetPrivilege(t)
 
-	tmpdir, err := os.MkdirTemp("", "evalrelative")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tmpdir)
+	tmpdir := t.TempDir()
 
 	if err := Touch(tmpdir); err == nil {
 		t.Errorf("touch can't take a directory")
@@ -490,14 +463,10 @@ func testCopyFileFunc(t *testing.T, fn copyFileFunc) {
 
 	testData := []byte("Hello, Singularity!")
 
-	tmpDir, err := os.MkdirTemp("", "copy-file")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tmpDir)
+	tmpDir := t.TempDir()
 
 	source := filepath.Join(tmpDir, "source")
-	err = os.WriteFile(source, testData, 0o644)
+	err := os.WriteFile(source, testData, 0o644)
 	if err != nil {
 		t.Fatalf("failed to create test source file: %v", err)
 	}
@@ -599,11 +568,7 @@ func TestIsWritable(t *testing.T) {
 	defer test.ResetPrivilege(t)
 
 	// We make a temporary directory where all the different cases will be tested.
-	tempDir, err := os.MkdirTemp("", "")
-	if err != nil {
-		t.Fatalf("failed to create temporary directory: %s", err)
-	}
-	defer os.RemoveAll(tempDir)
+	tempDir := t.TempDir()
 
 	// All the directory that we are about to create will be deleted when the temporary
 	// directory will be removed.
